#!/usr/bin/env ruby

###
## Copyright 2009 by Adron Systems LLC, Laporte, MN  56461.
## Permission is granted to copy, distribute and/or modify this document.
##
##   Purpose: Convert GC/MS or LC/MS file over to Adron text format (ASC).
##            Output is sent to stdout or output file.
##
##  Location: Adron\Engine\Macros.Ruby
##  Filename: AdronText.rb
##
##   Started: 2009/06/01
##   Updated:[2016/04/01]
###

# Load the Vx Extract interface library.
require 'ExtractLoad'


###
## Routine displayHelp :
###

def displayHelp
  system("cls")
  printf("Convert GC/MS file to Adron text format.  (Extract release : %s)\n", ASE::EXTRACT_RELEASE) ;
  puts
  printf("  %s [InFileName] [OutFileName] \n", File.basename($0))
  puts
  print("Options:\n")
  print("  InFileName   File to read (input) is first name specified.\n")
  print("  OutFileName  Output file is second name specified.\n")
  print("               If name not specified, then output goes to stdout.\n")
  puts
  print("  Optional items shown in [].\n")
  print("  No parameters generates this help screen.\n")
  puts
  print(ASE::ADRON_COPYRIGHT)
end


###
## Routine displayComments :
###

def displayComments( rdr, fout )
  fn = File.expand_path(rdr.nameOfFile)
  fn.gsub!( File::SEPARATOR, File::ALT_SEPARATOR )
  fout.printf("## Source file  -- %s\n", fn)
  fout.printf("## Vx Extract   -- %s\n", ASE::EXTRACT_RELEASE)
  fout.printf("## Scan Count   -- %d\n", rdr.numberOfScans)

  tm_now = ASE::Time.new
  fout.printf("## Generated on   %s UTC\n", tm_now.image(ASE::Time.FORMAT_VECTOR2_DATETIME))

  tm_inj = rdr.timeOfInjection
  fout.printf("## Injection time %s UTC\n", tm_inj.image(ASE::Time.FORMAT_VECTOR2_DATETIME))
end


###
## Routine displayFields :
###

def displayFields( rdr, fout )
  tm_inj = rdr.timeOfInjection
# fout.printf("TimeOfInjection=%d\n", tm_inj.ctime)
  fout.printf("NameOfOperator=%s\n", rdr.nameOfOperator)
  fout.printf("NameOfInstrument=%s\n", rdr.nameOfInstrument)
  fout.printf("NameOfSample=%s\n", rdr.nameOfSample)
  fout.printf("Description=%s\n", rdr.description)
  fout.printf("VialNumber=%d\n", rdr.vialNumber)
end


###
## Routine displaySpectrum :
##
##      Each spectrum is introduced with its scan interval value.
##      A listing of mass intensity pairs follows.
###

def displaySpectrum( rdr, nominal, i, spectrum, fout )
  # Extract scan information.
  peak_base = rdr.basePeakOfScan(nominal, i)
  itv_scan  = rdr.intervalOfScan( i )
  cnt_peaks = rdr.extractScan(nominal, i, spectrum)

  # Output interval of scan in seconds.
  fout.printf("IntervalOfScan=%.6f\n", itv_scan )

  # Generate format string for mass intensity pairs.
  str_frmt = "%d,%d\n"
  if (!nominal)
    mass_prec = (Math.log10(ASE::MI::MASS_SCALAR)).to_i
    str_frmt = sprintf("%%.%df,%%d\n", mass_prec)
  end

  # Output mass / intensity pairs as CSV.
  rg_pairs = (0...cnt_peaks)
  rg_pairs.each do |j|
    fout.printf(str_frmt, spectrum[j].mass, spectrum[j].intensity)
  end
end


###
## Entrance :
###

# No arguments, then display help screen.
if (ARGV.empty?)
  displayHelp
  Process.exit
end

# Open a ASE::Reader object.
begin
  rdr = ASE::Reader.new( ARGV[0] )
rescue Exception
  raise "Unable to create ASE::Reader object."
end

# Second argument is optional output file name.
fout = (nil != ARGV[1]) ? File.open(ARGV[1], "w+") : $stdout

# Display various comments.
displayComments( rdr, fout )
fout.puts

# Display file information fields.
displayFields( rdr, fout )
fout.puts
fout.puts

# Spectrum storage buffer.
spectrum=Array.new(50)

# Nominal (integer) mass versus experimental - boolean.
nominal = false

# Loop through all scans.
rg_scans = (0...rdr.numberOfScans)
rg_scans.each do |i|
  # Display the spectrum.
  displaySpectrum( rdr, nominal, i, spectrum, fout )

  # Do a spectrum separator character.
  if ( i != rg_scans.last )
    fout.puts
  end
end
